% Circle Packing Matlab Code
% Generates APM file and random csv for circle starts
% run analysis via APM server
% plots circles inscribed in circle
% Author: Jason Lund
% Date: 02/22/2013


% Circle Packing APM Generator
clc; clear all; clf; close all;

% important values
n = 10;
s = 10;
xlo = -2.0;
xhi = 2.0;
ylo = -2.0;
yhi = 2.0;
iters = 500;

fname = 'PCirMult.apm';
fid = fopen(fname,'w');            %# Open the file

% write APM file title
fprintf(fid,'!Pack Circles Optimization Problem \n!By: Jason Lund \n!Date:02/19/2013 \n \n');

% write constants section
fprintf(fid,'\n\tConstants \n');
fprintf(fid, '\t\tn = %1.0f \n',n);
fprintf(fid, '\t\ts = %1.0f \n',s);
fprintf(fid,'\tEnd Constants \n');

% write variables section
fprintf(fid, '\n\tVariables\n');
fprintf(fid, '\t\tx[1:n][1::s] >= %1.1f, <= %1.1f \n', -2.0, 2.0);
fprintf(fid, '\t\ty[1:n][1::s] >= %1.1f, <= %1.1f \n', -2.0, 2.0);
fprintf(fid, '\t\tcircrad[1::s] = %1.1f, >=%1.1f, <= %1.1f \n', 2.0, 1.7, 2.5);
fprintf(fid, '\tEnd Variables\n');

% write intermediates section
fprintf(fid, '\n\tIntermediates\n');
fprintf(fid, '\t\tints[1] = 1\n');
fprintf(fid, '\t\tints[2:n] = ints[1:n-1] + 1\n');
fprintf(fid, '\t\t! radii of circles\n');
fprintf(fid, '\t\tr[1:n] = ints[1:n]**(-0.5)\n');
fprintf(fid, '\tEnd Intermediates\n');

% write equations section
fprintf(fid, '\n\tEquations\n');
fprintf(fid, '\t\t! distance between pairs of circles is >= sum of radii\n');
for j = n:-1:2
    for k = j-1:-1:1
    fprintf(fid, '\t\tsqrt((x[%1.0f][1::s] - x[%1.0f][1::s])^2 + (y[%1.0f][1::s] - y[%1.0f][1::s])^2) >= r[%1.0f] + r[%1.0f]\n',j,k,j,k,j,k);
    end
end
fprintf(fid,'\t\t! circles are inside circumscribing circle\n');
fprintf(fid,'\t\tcircrad[1::s] - sqrt(x[1:n][1::s]^2 + y[1:n][1::s]^2) >= r[1:n]\n');
fprintf(fid, '\tminimize circrad[1::s]\n');
fprintf(fid, '\tEnd Equations\n');

fprintf(fid, '\nEnd Model');


% Write CSV file for start values
fprintf(fid, '\nFile *,csv\n');
xr = zeros(n,1);
yr = xr;
for j = 1:s
    for i = 1:n
        xr(i) = 10;
        yr(i) = 10;
        fprintf(fid, 'packcircles.x[%1.0f][%1.0f], %1f\n', i,j, xr(i));
        fprintf(fid, 'packcircles.y[%1.0f][%1.0f], %1f\n', i,j, yr(i));
    end
end
fprintf(fid, '\nEnd File\n');


%load the APM MATLAB library
addpath('apm')

%select the server
server = 'http://byu.apmonitor.com';%'http://apmonitor.com/online/view_pass.php';

%give the application a name
app = 'packcircles';

%clear any previous applications by that name
apm(server,app,'clear all');

%load the model file
apm_load(server,app,fname);
apm_option(server,app,'nlc.solver',1);
apm_option(server,app,'nlc.coldstart',1);
apm_option(server,app,'nlc.max_iter',iters);

%solve the optimization problem
apm(server,app,'solve');

%retrieve the solution
y = apm_sol(server,app);
z = y.x;


%%
% evaluate and plot results
cRad = zeros(s,1);
for k = 1:s
   cRad(k) = eval(['z.circrad' int2str(k)]);
end
[cBest,Ibest] = min(cRad);
[cWorst,Iworst] = max(cRad);

%plot results
cb = zeros(n+1,2);
cw = cb;
rb = zeros(n+1,1);
rw = rb;
for k = 1:n
   cb(k,:) = [eval(['z.x' int2str(k) int2str(Ibest)]), eval(['z.y' int2str(k) int2str(Ibest)])];
   rb(k) = 1/sqrt(k);
   cw(k,:) = [eval(['z.x' int2str(k) int2str(Iworst)]), eval(['z.y' int2str(k) int2str(Iworst)])];
   rw(k) = 1/sqrt(k);
end
rb(n+1) = cBest;
rw(n+1) = cWorst;

% Output blurb about results
disp('Solution Found!')
disp(['Best Circle Radius: ' num2str(cBest)])
disp(['Worst Circle Radius: ' num2str(cWorst)])
% disp('Circle Coordinate Values:')
% for i = 1:n
%     disp(['Circle_' num2str(i) ':   ' num2str(c(i,1)) '    ' num2str(c(i,2))])
% end


figure(1)
subplot(1,2,1)
hold on;
for k = 1:n+1
[h,xs,ys] = circles(rb(k),cb(k,:),...
          'color','g','linewidth',1.5);   
end
labels = cellstr( num2str([1:n]') );  %' # labels correspond to their order
text(cb(1:n,1), cb(1:n,2), labels, 'VerticalAlignment','bottom', ...
                             'HorizontalAlignment','left');
axis equal
axis([-3 3 -3 3])
title(['Best Optimum: ' num2str(cBest)])


subplot(1,2,2)
for k = 1:n+1
[h,xs,ys] = circles(rw(k),cw(k,:),...
          'color','r','linewidth',1.5);   
end
labels = cellstr( num2str([1:n]') );  %' # labels correspond to their order
text(cw(1:n,1), cw(1:n,2), labels, 'VerticalAlignment','bottom', ...
                             'HorizontalAlignment','left');
axis equal
axis([-3 3 -3 3])
title(['Worst Optimum: ' num2str(cWorst)])


% NExt, add loop to rewrite APM file to add new search directions 